"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var core_1 = require("@angular/core");
var forms_1 = require("@angular/forms");
var Rx_1 = require("rxjs/Rx");
var ElasticDirective = (function () {
    function ElasticDirective(element, ngZone, model) {
        this.element = element;
        this.ngZone = ngZone;
        this.model = model;
        this.previousScrollHeight = 0;
        if (!model) {
            return;
        }
        // Hijack NgModels ControlValueAccessor to detect when model is updated.
        var self = this;
        var originalWriteValue = model.valueAccessor.writeValue;
        function writeValue(obj) {
            if (originalWriteValue) {
                // Invoke the original writeValue function.
                originalWriteValue.call(this, obj);
            }
            // Adjust the textarea size.
            self.adjust();
        }
        model.valueAccessor.writeValue = writeValue;
    }
    ElasticDirective.prototype.isTextarea = function (el) {
        return el.tagName === 'TEXTAREA';
    };
    ElasticDirective.prototype.setupTextarea = function (textareaEl) {
        var _this = this;
        this.textareaEl = textareaEl;
        var style = this.textareaEl.style;
        style.overflow = 'hidden';
        style.resize = 'none';
        this.ngZone.runOutsideAngular(function () {
            Rx_1.Observable.fromEvent(window, 'resize')
                .debounceTime(100)
                .subscribe(function () { return _this.adjust(); });
        });
    };
    ElasticDirective.prototype.ngAfterViewInit = function () {
        var _this = this;
        if (this.isTextarea(this.element.nativeElement)) {
            this.setupTextarea(this.element.nativeElement);
            return;
        }
        var children = Array.from(this.element.nativeElement.children);
        var textareaEl = children.find(function (el) { return _this.isTextarea(el); });
        if (textareaEl) {
            this.setupTextarea(textareaEl);
            return;
        }
        throw new Error('The `elastic` attribute directive must be used on a `textarea` or an element that contains a `textarea`.');
    };
    ElasticDirective.prototype.onInput = function () {
        this.adjust();
    };
    ElasticDirective.prototype.adjust = function () {
        if (!this.textareaEl || this.previousScrollHeight === this.textareaEl.scrollHeight) {
            return;
        }
        this.textareaEl.style.height = 'auto';
        this.textareaEl.style.height = this.textareaEl.scrollHeight + "px";
        this.previousScrollHeight = this.textareaEl.scrollHeight;
    };
    return ElasticDirective;
}());
ElasticDirective.decorators = [
    { type: core_1.Directive, args: [{
                selector: '[fz-elastic]'
            },] },
];
/** @nocollapse */
ElasticDirective.ctorParameters = function () { return [
    { type: core_1.ElementRef, },
    { type: core_1.NgZone, },
    { type: forms_1.NgModel, decorators: [{ type: core_1.Optional },] },
]; };
ElasticDirective.propDecorators = {
    'onInput': [{ type: core_1.HostListener, args: ['input',] },],
};
exports.ElasticDirective = ElasticDirective;
